@@ -27,7 +27,9 @@ from django.db.backends import BaseDatabaseWrapper, BaseDatabaseFeatures, BaseDa
 from django.db.backends.signals import connection_created
 from django.conf import settings
 from django import VERSION as DjangoVersion
-if DjangoVersion[:2] >= (1,5):
+if DjangoVersion[:2] == (1,6):
+    _DJANGO_VERSION = 16
+elif DjangoVersion[:2] == (1,5):
     _DJANGO_VERSION = 15
 elif DjangoVersion[:2] == (1,4):
     _DJANGO_VERSION = 14
@@ -55,6 +57,17 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_subqueries_in_group_by = False
     supports_transactions = True
     #uses_savepoints = True
+    allow_sliced_subqueries = False
+    supports_paramstyle_pyformat = False
+
+    #has_bulk_insert = False
+    # DateTimeField doesn't support timezones, only DateTimeOffsetField
+    supports_timezones = False
+    supports_sequence_reset = False    
+    supports_tablespaces = True
+    ignores_nulls_in_unique_constraints = False
+    can_introspect_autofield = True
+
 
     def _supports_transactions(self):
         # keep it compatible with Django 1.3 and 1.4
@@ -67,6 +80,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     MARS_Connection = False
     unicode_results = False
     datefirst = 7
+    Database = Database
 
     # Collations:       http://msdn2.microsoft.com/en-us/library/ms184391.aspx
     #                   http://msdn2.microsoft.com/en-us/library/ms179886.aspx
@@ -139,11 +153,41 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
         self.connection = None
 
-    def _cursor(self):
-        new_conn = False
+    def get_connection_params(self):
+        settings_dict = self.settings_dict
+        if not settings_dict['NAME']:
+            from django.core.exceptions import ImproperlyConfigured
+            raise ImproperlyConfigured(
+                "settings.DATABASES is improperly configured. "
+                "Please supply the NAME value.")
+        conn_params = {
+            'database': settings_dict['NAME'],
+        }
+        conn_params.update(settings_dict['OPTIONS'])
+        if 'autocommit' in conn_params:
+            del conn_params['autocommit']
+        if settings_dict['USER']:
+            conn_params['user'] = settings_dict['USER']
+        if settings_dict['PASSWORD']:
+            conn_params['password'] = settings_dict['PASSWORD']
+        if settings_dict['HOST']:
+            conn_params['host'] = settings_dict['HOST']
+        if settings_dict['PORT']:
+            conn_params['port'] = settings_dict['PORT']
+        return conn_params
+
+    def get_new_connection(self, conn_params):
+        return Database.connect(**conn_params)
+
+    def init_connection_state(self):
+        pass
+
+    def _set_autocommit(self, autocommit):
+        pass
+
+    def _get_connection_string(self):
         settings_dict = self.settings_dict
         db_str, user_str, passwd_str, port_str = None, None, "", None
-
         options = settings_dict['OPTIONS']
         if settings_dict['NAME']:
             db_str = settings_dict['NAME']
@@ -158,66 +202,74 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         if settings_dict['PORT']:
             port_str = settings_dict['PORT']
 
+        if not db_str:
+            raise ImproperlyConfigured('You need to specify NAME in your Django settings file.')
+
+        cstr_parts = []
+        if 'driver' in options:
+            driver = options['driver']
+        else:
+            if os.name == 'nt':
+                driver = 'SQL Server'
+            else:
+                driver = 'FreeTDS'
+
+        if driver == 'FreeTDS' or driver.endswith('/libtdsodbc.so'):
+            driver_is_freetds = True
+        else:
+            driver_is_freetds = False
+
+        # Microsoft driver names assumed here are:
+        # * SQL Server
+        # * SQL Native Client
+        # * SQL Server Native Client 10.0/11.0
+        # * ODBC Driver 11 for SQL Server
+        ms_drivers = re.compile('.*SQL (Server$|(Server )?Native Client)')
+
+        if 'dsn' in options:
+            cstr_parts.append('DSN=%s' % options['dsn'])
+        else:
+            # Only append DRIVER if DATABASE_ODBC_DSN hasn't been set
+            if os.path.isabs(driver):
+                cstr_parts.append('DRIVER=%s' % driver)
+            else:
+                cstr_parts.append('DRIVER={%s}' % driver)
+
+            if ms_drivers.match(driver) or driver_is_freetds and \
+                    options.get('host_is_server', False):
+                if port_str:
+                    host_str += ';PORT=%s' % port_str
+                cstr_parts.append('SERVER=%s' % host_str)
+            else:
+                cstr_parts.append('SERVERNAME=%s' % host_str)
+
+        if user_str:
+            cstr_parts.append('UID=%s;PWD=%s' % (user_str, passwd_str))
+        else:
+            if ms_drivers.match(driver):
+                cstr_parts.append('Trusted_Connection=yes')
+            else:
+                cstr_parts.append('Integrated Security=SSPI')
+
+        cstr_parts.append('DATABASE=%s' % db_str)
+
+        if self.MARS_Connection:
+            cstr_parts.append('MARS_Connection=yes')
+
+        if 'extra_params' in options:
+            cstr_parts.append(options['extra_params'])
+        connectionstring = ';'.join(cstr_parts)
+        return connectionstring
+
+    def _cursor(self):
+        new_conn = False
+        settings_dict = self.settings_dict
+
+
         if self.connection is None:
             new_conn = True
-            if not db_str:
-                raise ImproperlyConfigured('You need to specify NAME in your Django settings file.')
-
-            cstr_parts = []
-            if 'driver' in options:
-                driver = options['driver']
-            else:
-                if os.name == 'nt':
-                    driver = 'SQL Server'
-                else:
-                    driver = 'FreeTDS'
-
-            if driver == 'FreeTDS' or driver.endswith('/libtdsodbc.so'):
-                driver_is_freetds = True
-            else:
-                driver_is_freetds = False
-
-            # Microsoft driver names assumed here are:
-            # * SQL Server
-            # * SQL Native Client
-            # * SQL Server Native Client 10.0/11.0
-            # * ODBC Driver 11 for SQL Server
-            ms_drivers = re.compile('.*SQL (Server$|(Server )?Native Client)')
-
-            if 'dsn' in options:
-                cstr_parts.append('DSN=%s' % options['dsn'])
-            else:
-                # Only append DRIVER if DATABASE_ODBC_DSN hasn't been set
-                if os.path.isabs(driver):
-                    cstr_parts.append('DRIVER=%s' % driver)
-                else:
-                    cstr_parts.append('DRIVER={%s}' % driver)
-
-                if ms_drivers.match(driver) or driver_is_freetds and \
-                        options.get('host_is_server', False):
-                    if port_str:
-                        host_str += ';PORT=%s' % port_str
-                    cstr_parts.append('SERVER=%s' % host_str)
-                else:
-                    cstr_parts.append('SERVERNAME=%s' % host_str)
-
-            if user_str:
-                cstr_parts.append('UID=%s;PWD=%s' % (user_str, passwd_str))
-            else:
-                if ms_drivers.match(driver):
-                    cstr_parts.append('Trusted_Connection=yes')
-                else:
-                    cstr_parts.append('Integrated Security=SSPI')
-
-            cstr_parts.append('DATABASE=%s' % db_str)
-
-            if self.MARS_Connection:
-                cstr_parts.append('MARS_Connection=yes')
-
-            if 'extra_params' in options:
-                cstr_parts.append(options['extra_params'])
-
-            connstr = ';'.join(cstr_parts)
+            connstr = self._get_connection_string()#';'.join(cstr_parts)
+            options = settings_dict['OPTIONS']
             autocommit = options.get('autocommit', False)
             if self.unicode_results:
                 self.connection = Database.connect(connstr, \
@@ -315,7 +367,11 @@ class CursorWrapper(object):
             sql = sql.encode('utf-8')
         # pyodbc uses '?' instead of '%s' as parameter placeholder.
         if n_params is not None:
-            sql = sql % tuple('?' * n_params)
+            try:
+                sql = sql % tuple('?' * n_params)
+            except:
+                #Todo checkout whats happening here
+                pass
         else:
             if '%s' in sql:
                 sql = sql.replace('%s', '?')
@@ -350,7 +406,6 @@ class CursorWrapper(object):
         sql = self.format_sql(sql, len(params))
         params = self.format_params(params)
         self.last_params = params
-
         try:
             return self.cursor.execute(sql, params)
         except IntegrityError:
@@ -418,3 +473,16 @@ class CursorWrapper(object):
 
     def __iter__(self):
         return iter(self.cursor)
+
+
+    # # MS SQL Server doesn't support explicit savepoint commits; savepoints are
+    # # implicitly committed with the transaction.
+    # # Ignore them.
+    def savepoint_commit(self, sid):
+        # if something is populating self.queries, include a fake entry to avoid
+        # issues with tests that use assertNumQueries.
+        if self.queries:
+            self.queries.append({
+                'sql': '-- RELEASE SAVEPOINT %s -- (because assertNumQueries)' % self.ops.quote_name(sid),
+                'time': '0.000',
+            })
\ No newline at end of file
